# Copyright 2021 The KCL Authors. All rights reserved.

import os
import typing
import sys
import subprocess
import glob
import platform

import kclvm.config
import kclvm.kcl.ast as ast
import kclvm.api.object as objpkg
import kclvm.compiler.build.compiler as compiler

from .native_runner import (
    LL_FILE_PATTERN,
    native_run_dylib,
)


def native_run_windows(
    path_list: typing.List[str], *, ast_prog: ast.Program
) -> objpkg.KCLResult:
    if platform.system() != "Windows":
        raise "native_run_windows only for windows"

    from kclvm.internal.kclx.transformer import transform_ast_to_kclx_ast_json_str

    # Config

    _no_link = True
    _executable_root = os.path.dirname(sys.executable)
    _kclvm_cli = f"{_executable_root}\\kclvm-cli.exe"
    _clang = f"{_executable_root}\\tools\\clang\\bin\\clang.exe"
    _clang = _clang if os.path.exists(_clang) else "clang.exe"

    _kclvm_main_win_c = f"{_executable_root}\\libs\\_kclvm_main_win.c"
    _kclvm_dll_lib = f"{_executable_root}\\libs\\kclvm.dll.lib"
    _kclvm_bc = f"{_executable_root}\\libs\\_kclvm.bc"
    _a_out_ast_json = "_a.out.ast.json"
    _a_out_ll = "_a.out.ll"
    _a_out_dylib = "_a.out.dll"
    _out_bc_files = []

    # Resolve Program
    compiler.CompileProgram(ast_prog)

    # Build Program with kclvm-cli, windows donot support cache

    if True:
        # Transfrom Program
        ast_json = transform_ast_to_kclx_ast_json_str(ast_prog)
        with open(_a_out_ast_json, "w") as file:
            file.write(ast_json)
        if _no_link and os.path.exists(_a_out_ll):
            os.remove(_a_out_ll)
        if _no_link:
            _out_bc_files = glob.glob(_a_out_ll + LL_FILE_PATTERN)
            for file in _out_bc_files:
                if os.path.exists(file):
                    os.remove(file)

        # kclvm compile
        try:
            args = [
                _kclvm_cli,
                "build",
                _a_out_ast_json,
                "--bc",
                _kclvm_bc,
                "-o",
                _a_out_ll + ".tmp.ll",
            ]
            subprocess.check_call(args)

            fix_ll_local_name(_a_out_ll, _a_out_ll + ".tmp.ll")
            os.remove(_a_out_ll + ".tmp.ll")

        except subprocess.CalledProcessError as e:
            raise e
        _out_bc_files = glob.glob(_a_out_ll + LL_FILE_PATTERN)

        # clang
        try:
            args = [
                _clang,
                "-Wno-override-module",
                "-shared",
                _a_out_ll,
                _kclvm_main_win_c,
                _kclvm_dll_lib,
                "-lws2_32",
                "-lbcrypt",
                "-lAdvapi32",
                "-lUserenv",
                "-o",
                _a_out_dylib,
            ]
            subprocess.check_call(args, stdout=open(os.devnull, "wb"))

        except subprocess.CalledProcessError as e:
            raise e

    # run app
    result = native_run_dylib(path_list, _a_out_dylib)
    if not kclvm.config.debug:
        if os.path.exists(_a_out_ast_json):
            os.remove(_a_out_ast_json)
        if _no_link:
            for file in _out_bc_files:
                if os.path.exists(file):
                    os.remove(file)
        else:
            if os.path.exists(_a_out_ll):
                os.remove(_a_out_ll)
        if os.path.exists(_a_out_dylib):
            os.remove(_a_out_dylib)

    return result


def fix_ll_local_name(dst_path, src_path: str):
    replaceArgs_old = []
    replaceArgs_new = []

    for i in range(0, 10):
        replaceArgs_old.append(f"%{i}")
        replaceArgs_new.append(f"%local_{i}")

        replaceArgs_old.append(f"\n{i}")
        replaceArgs_new.append(f"\nlocal_{i}")

    with open(src_path, "r") as file:
        code = file.read()

        for i in range(0, len(replaceArgs_old)):
            code = code.replace(replaceArgs_old[i], replaceArgs_new[i], -1)

        with open(dst_path, "w") as f:
            f.write(code)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: model.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='model.proto',
  package='kclvm.doc.model',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0bmodel.proto\x12\x0fkclvm.doc.model\"\x83\x01\n\tModuleDoc\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x15\n\rrelative_path\x18\x02 \x01(\t\x12\x0b\n\x03\x64oc\x18\x03 \x01(\t\x12+\n\x07schemas\x18\x04 \x03(\x0b\x32\x1a.kclvm.doc.model.SchemaDoc\x12\x17\n\x0fsource_code_url\x18\x05 \x01(\t\"\x9d\x01\n\tSchemaDoc\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0b\n\x03\x64oc\x18\x02 \x01(\t\x12\x37\n\nattributes\x18\x03 \x03(\x0b\x32#.kclvm.doc.model.SchemaAttributeDoc\x12\x10\n\x08\x65xamples\x18\x04 \x01(\t\x12*\n\x0b\x62\x61se_schema\x18\x05 \x01(\x0b\x32\x15.kclvm.doc.model.Type\"\x80\x01\n\x12SchemaAttributeDoc\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0b\n\x03\x64oc\x18\x02 \x01(\t\x12#\n\x04type\x18\x04 \x01(\x0b\x32\x15.kclvm.doc.model.Type\x12\x13\n\x0bis_optional\x18\x05 \x01(\x08\x12\x15\n\rdefault_value\x18\x06 \x01(\t\"\x9c\x06\n\x04Type\x12\x10\n\x08type_str\x18\x01 \x01(\t\x12\x39\n\rtype_category\x18\x02 \x01(\x0e\x32\".kclvm.doc.model.Type.TypeCategory\x12\x33\n\nunion_type\x18\x03 \x01(\x0b\x32\x1a.kclvm.doc.model.UnionTypeH\x00\x88\x01\x01\x12\x31\n\tlist_type\x18\x04 \x01(\x0b\x32\x19.kclvm.doc.model.ListTypeH\x01\x88\x01\x01\x12\x31\n\tdict_type\x18\x05 \x01(\x0b\x32\x19.kclvm.doc.model.DictTypeH\x02\x88\x01\x01\x12\x35\n\x0bschema_type\x18\x06 \x01(\x0b\x32\x1b.kclvm.doc.model.SchemaTypeH\x03\x88\x01\x01\x12<\n\x0c\x62uiltin_type\x18\x07 \x01(\x0e\x32!.kclvm.doc.model.Type.BuiltinTypeH\x04\x88\x01\x01\x12/\n\x08lit_type\x18\x08 \x01(\x0b\x32\x18.kclvm.doc.model.LitTypeH\x05\x88\x01\x01\x12@\n\x0enamed_constant\x18\t \x01(\x0e\x32#.kclvm.doc.model.Type.NamedConstantH\x06\x88\x01\x01\"\x83\x01\n\x0cTypeCategory\x12\t\n\x05UNION\x10\x00\x12\x08\n\x04LIST\x10\x01\x12\x08\n\x04\x44ICT\x10\x02\x12\n\n\x06SCHEMA\x10\x03\x12\x0b\n\x07\x42UILTIN\x10\x04\x12\x07\n\x03LIT\x10\x05\x12\x12\n\x0eNAMED_CONSTANT\x10\x06\x12\x07\n\x03\x41NY\x10\x07\x12\x15\n\x11NUMBER_MULTIPLIER\x10\x08\"7\n\x0b\x42uiltinType\x12\x07\n\x03INT\x10\x00\x12\t\n\x05\x46LOAT\x10\x01\x12\n\n\x06STRING\x10\x02\x12\x08\n\x04\x42OOL\x10\x03\"\x19\n\rNamedConstant\x12\x08\n\x04NONE\x10\x00\x42\r\n\x0b_union_typeB\x0c\n\n_list_typeB\x0c\n\n_dict_typeB\x0e\n\x0c_schema_typeB\x0f\n\r_builtin_typeB\x0b\n\t_lit_typeB\x11\n\x0f_named_constant\"1\n\tUnionType\x12$\n\x05types\x18\x01 \x03(\x0b\x32\x15.kclvm.doc.model.Type\"4\n\x08ListType\x12(\n\titem_type\x18\x01 \x01(\x0b\x32\x15.kclvm.doc.model.Type\"^\n\x08\x44ictType\x12\'\n\x08key_type\x18\x01 \x01(\x0b\x32\x15.kclvm.doc.model.Type\x12)\n\nvalue_type\x18\x02 \x01(\x0b\x32\x15.kclvm.doc.model.Type\"1\n\nSchemaType\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x15\n\rrelative_path\x18\x02 \x01(\t\"\x9d\x01\n\x07LitType\x12\x17\n\nstring_lit\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x14\n\x07int_lit\x18\x02 \x01(\x03H\x01\x88\x01\x01\x12\x16\n\tfloat_lit\x18\x03 \x01(\x02H\x02\x88\x01\x01\x12\x15\n\x08\x62ool_lit\x18\x04 \x01(\x08H\x03\x88\x01\x01\x42\r\n\x0b_string_litB\n\n\x08_int_litB\x0c\n\n_float_litB\x0b\n\t_bool_litb\x06proto3'
)



_TYPE_TYPECATEGORY = _descriptor.EnumDescriptor(
  name='TypeCategory',
  full_name='kclvm.doc.model.Type.TypeCategory',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNION', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LIST', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DICT', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SCHEMA', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BUILTIN', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LIT', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='NAMED_CONSTANT', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ANY', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='NUMBER_MULTIPLIER', index=8, number=8,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=931,
  serialized_end=1062,
)
_sym_db.RegisterEnumDescriptor(_TYPE_TYPECATEGORY)

_TYPE_BUILTINTYPE = _descriptor.EnumDescriptor(
  name='BuiltinType',
  full_name='kclvm.doc.model.Type.BuiltinType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='INT', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FLOAT', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='STRING', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BOOL', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1064,
  serialized_end=1119,
)
_sym_db.RegisterEnumDescriptor(_TYPE_BUILTINTYPE)

_TYPE_NAMEDCONSTANT = _descriptor.EnumDescriptor(
  name='NamedConstant',
  full_name='kclvm.doc.model.Type.NamedConstant',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NONE', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1121,
  serialized_end=1146,
)
_sym_db.RegisterEnumDescriptor(_TYPE_NAMEDCONSTANT)


_MODULEDOC = _descriptor.Descriptor(
  name='ModuleDoc',
  full_name='kclvm.doc.model.ModuleDoc',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='kclvm.doc.model.ModuleDoc.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='relative_path', full_name='kclvm.doc.model.ModuleDoc.relative_path', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='doc', full_name='kclvm.doc.model.ModuleDoc.doc', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='schemas', full_name='kclvm.doc.model.ModuleDoc.schemas', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='source_code_url', full_name='kclvm.doc.model.ModuleDoc.source_code_url', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=33,
  serialized_end=164,
)


_SCHEMADOC = _descriptor.Descriptor(
  name='SchemaDoc',
  full_name='kclvm.doc.model.SchemaDoc',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='kclvm.doc.model.SchemaDoc.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='doc', full_name='kclvm.doc.model.SchemaDoc.doc', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='attributes', full_name='kclvm.doc.model.SchemaDoc.attributes', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='examples', full_name='kclvm.doc.model.SchemaDoc.examples', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='base_schema', full_name='kclvm.doc.model.SchemaDoc.base_schema', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=167,
  serialized_end=324,
)


_SCHEMAATTRIBUTEDOC = _descriptor.Descriptor(
  name='SchemaAttributeDoc',
  full_name='kclvm.doc.model.SchemaAttributeDoc',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='kclvm.doc.model.SchemaAttributeDoc.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='doc', full_name='kclvm.doc.model.SchemaAttributeDoc.doc', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='kclvm.doc.model.SchemaAttributeDoc.type', index=2,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='is_optional', full_name='kclvm.doc.model.SchemaAttributeDoc.is_optional', index=3,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='default_value', full_name='kclvm.doc.model.SchemaAttributeDoc.default_value', index=4,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=327,
  serialized_end=455,
)


_TYPE = _descriptor.Descriptor(
  name='Type',
  full_name='kclvm.doc.model.Type',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type_str', full_name='kclvm.doc.model.Type.type_str', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type_category', full_name='kclvm.doc.model.Type.type_category', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='union_type', full_name='kclvm.doc.model.Type.union_type', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='list_type', full_name='kclvm.doc.model.Type.list_type', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='dict_type', full_name='kclvm.doc.model.Type.dict_type', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='schema_type', full_name='kclvm.doc.model.Type.schema_type', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='builtin_type', full_name='kclvm.doc.model.Type.builtin_type', index=6,
      number=7, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='lit_type', full_name='kclvm.doc.model.Type.lit_type', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='named_constant', full_name='kclvm.doc.model.Type.named_constant', index=8,
      number=9, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _TYPE_TYPECATEGORY,
    _TYPE_BUILTINTYPE,
    _TYPE_NAMEDCONSTANT,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='_union_type', full_name='kclvm.doc.model.Type._union_type',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
    _descriptor.OneofDescriptor(
      name='_list_type', full_name='kclvm.doc.model.Type._list_type',
      index=1, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
    _descriptor.OneofDescriptor(
      name='_dict_type', full_name='kclvm.doc.model.Type._dict_type',
      index=2, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
    _descriptor.OneofDescriptor(
      name='_schema_type', full_name='kclvm.doc.model.Type._schema_type',
      index=3, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
    _descriptor.OneofDescriptor(
      name='_builtin_type', full_name='kclvm.doc.model.Type._builtin_type',
      index=4, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
    _descriptor.OneofDescriptor(
      name='_lit_type', full_name='kclvm.doc.model.Type._lit_type',
      index=5, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
    _descriptor.OneofDescriptor(
      name='_named_constant', full_name='kclvm.doc.model.Type._named_constant',
      index=6, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=458,
  serialized_end=1254,
)


_UNIONTYPE = _descriptor.Descriptor(
  name='UnionType',
  full_name='kclvm.doc.model.UnionType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='types', full_name='kclvm.doc.model.UnionType.types', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1256,
  serialized_end=1305,
)


_LISTTYPE = _descriptor.Descriptor(
  name='ListType',
  full_name='kclvm.doc.model.ListType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='item_type', full_name='kclvm.doc.model.ListType.item_type', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1307,
  serialized_end=1359,
)


_DICTTYPE = _descriptor.Descriptor(
  name='DictType',
  full_name='kclvm.doc.model.DictType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key_type', full_name='kclvm.doc.model.DictType.key_type', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value_type', full_name='kclvm.doc.model.DictType.value_type', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1361,
  serialized_end=1455,
)


_SCHEMATYPE = _descriptor.Descriptor(
  name='SchemaType',
  full_name='kclvm.doc.model.SchemaType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='kclvm.doc.model.SchemaType.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='relative_path', full_name='kclvm.doc.model.SchemaType.relative_path', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1457,
  serialized_end=1506,
)


_LITTYPE = _descriptor.Descriptor(
  name='LitType',
  full_name='kclvm.doc.model.LitType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='string_lit', full_name='kclvm.doc.model.LitType.string_lit', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='int_lit', full_name='kclvm.doc.model.LitType.int_lit', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='float_lit', full_name='kclvm.doc.model.LitType.float_lit', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bool_lit', full_name='kclvm.doc.model.LitType.bool_lit', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='_string_lit', full_name='kclvm.doc.model.LitType._string_lit',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
    _descriptor.OneofDescriptor(
      name='_int_lit', full_name='kclvm.doc.model.LitType._int_lit',
      index=1, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
    _descriptor.OneofDescriptor(
      name='_float_lit', full_name='kclvm.doc.model.LitType._float_lit',
      index=2, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
    _descriptor.OneofDescriptor(
      name='_bool_lit', full_name='kclvm.doc.model.LitType._bool_lit',
      index=3, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1509,
  serialized_end=1666,
)

_MODULEDOC.fields_by_name['schemas'].message_type = _SCHEMADOC
_SCHEMADOC.fields_by_name['attributes'].message_type = _SCHEMAATTRIBUTEDOC
_SCHEMADOC.fields_by_name['base_schema'].message_type = _TYPE
_SCHEMAATTRIBUTEDOC.fields_by_name['type'].message_type = _TYPE
_TYPE.fields_by_name['type_category'].enum_type = _TYPE_TYPECATEGORY
_TYPE.fields_by_name['union_type'].message_type = _UNIONTYPE
_TYPE.fields_by_name['list_type'].message_type = _LISTTYPE
_TYPE.fields_by_name['dict_type'].message_type = _DICTTYPE
_TYPE.fields_by_name['schema_type'].message_type = _SCHEMATYPE
_TYPE.fields_by_name['builtin_type'].enum_type = _TYPE_BUILTINTYPE
_TYPE.fields_by_name['lit_type'].message_type = _LITTYPE
_TYPE.fields_by_name['named_constant'].enum_type = _TYPE_NAMEDCONSTANT
_TYPE_TYPECATEGORY.containing_type = _TYPE
_TYPE_BUILTINTYPE.containing_type = _TYPE
_TYPE_NAMEDCONSTANT.containing_type = _TYPE
_TYPE.oneofs_by_name['_union_type'].fields.append(
  _TYPE.fields_by_name['union_type'])
_TYPE.fields_by_name['union_type'].containing_oneof = _TYPE.oneofs_by_name['_union_type']
_TYPE.oneofs_by_name['_list_type'].fields.append(
  _TYPE.fields_by_name['list_type'])
_TYPE.fields_by_name['list_type'].containing_oneof = _TYPE.oneofs_by_name['_list_type']
_TYPE.oneofs_by_name['_dict_type'].fields.append(
  _TYPE.fields_by_name['dict_type'])
_TYPE.fields_by_name['dict_type'].containing_oneof = _TYPE.oneofs_by_name['_dict_type']
_TYPE.oneofs_by_name['_schema_type'].fields.append(
  _TYPE.fields_by_name['schema_type'])
_TYPE.fields_by_name['schema_type'].containing_oneof = _TYPE.oneofs_by_name['_schema_type']
_TYPE.oneofs_by_name['_builtin_type'].fields.append(
  _TYPE.fields_by_name['builtin_type'])
_TYPE.fields_by_name['builtin_type'].containing_oneof = _TYPE.oneofs_by_name['_builtin_type']
_TYPE.oneofs_by_name['_lit_type'].fields.append(
  _TYPE.fields_by_name['lit_type'])
_TYPE.fields_by_name['lit_type'].containing_oneof = _TYPE.oneofs_by_name['_lit_type']
_TYPE.oneofs_by_name['_named_constant'].fields.append(
  _TYPE.fields_by_name['named_constant'])
_TYPE.fields_by_name['named_constant'].containing_oneof = _TYPE.oneofs_by_name['_named_constant']
_UNIONTYPE.fields_by_name['types'].message_type = _TYPE
_LISTTYPE.fields_by_name['item_type'].message_type = _TYPE
_DICTTYPE.fields_by_name['key_type'].message_type = _TYPE
_DICTTYPE.fields_by_name['value_type'].message_type = _TYPE
_LITTYPE.oneofs_by_name['_string_lit'].fields.append(
  _LITTYPE.fields_by_name['string_lit'])
_LITTYPE.fields_by_name['string_lit'].containing_oneof = _LITTYPE.oneofs_by_name['_string_lit']
_LITTYPE.oneofs_by_name['_int_lit'].fields.append(
  _LITTYPE.fields_by_name['int_lit'])
_LITTYPE.fields_by_name['int_lit'].containing_oneof = _LITTYPE.oneofs_by_name['_int_lit']
_LITTYPE.oneofs_by_name['_float_lit'].fields.append(
  _LITTYPE.fields_by_name['float_lit'])
_LITTYPE.fields_by_name['float_lit'].containing_oneof = _LITTYPE.oneofs_by_name['_float_lit']
_LITTYPE.oneofs_by_name['_bool_lit'].fields.append(
  _LITTYPE.fields_by_name['bool_lit'])
_LITTYPE.fields_by_name['bool_lit'].containing_oneof = _LITTYPE.oneofs_by_name['_bool_lit']
DESCRIPTOR.message_types_by_name['ModuleDoc'] = _MODULEDOC
DESCRIPTOR.message_types_by_name['SchemaDoc'] = _SCHEMADOC
DESCRIPTOR.message_types_by_name['SchemaAttributeDoc'] = _SCHEMAATTRIBUTEDOC
DESCRIPTOR.message_types_by_name['Type'] = _TYPE
DESCRIPTOR.message_types_by_name['UnionType'] = _UNIONTYPE
DESCRIPTOR.message_types_by_name['ListType'] = _LISTTYPE
DESCRIPTOR.message_types_by_name['DictType'] = _DICTTYPE
DESCRIPTOR.message_types_by_name['SchemaType'] = _SCHEMATYPE
DESCRIPTOR.message_types_by_name['LitType'] = _LITTYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ModuleDoc = _reflection.GeneratedProtocolMessageType('ModuleDoc', (_message.Message,), {
  'DESCRIPTOR' : _MODULEDOC,
  '__module__' : 'model_pb2'
  # @@protoc_insertion_point(class_scope:kclvm.doc.model.ModuleDoc)
  })
_sym_db.RegisterMessage(ModuleDoc)

SchemaDoc = _reflection.GeneratedProtocolMessageType('SchemaDoc', (_message.Message,), {
  'DESCRIPTOR' : _SCHEMADOC,
  '__module__' : 'model_pb2'
  # @@protoc_insertion_point(class_scope:kclvm.doc.model.SchemaDoc)
  })
_sym_db.RegisterMessage(SchemaDoc)

SchemaAttributeDoc = _reflection.GeneratedProtocolMessageType('SchemaAttributeDoc', (_message.Message,), {
  'DESCRIPTOR' : _SCHEMAATTRIBUTEDOC,
  '__module__' : 'model_pb2'
  # @@protoc_insertion_point(class_scope:kclvm.doc.model.SchemaAttributeDoc)
  })
_sym_db.RegisterMessage(SchemaAttributeDoc)

Type = _reflection.GeneratedProtocolMessageType('Type', (_message.Message,), {
  'DESCRIPTOR' : _TYPE,
  '__module__' : 'model_pb2'
  # @@protoc_insertion_point(class_scope:kclvm.doc.model.Type)
  })
_sym_db.RegisterMessage(Type)

UnionType = _reflection.GeneratedProtocolMessageType('UnionType', (_message.Message,), {
  'DESCRIPTOR' : _UNIONTYPE,
  '__module__' : 'model_pb2'
  # @@protoc_insertion_point(class_scope:kclvm.doc.model.UnionType)
  })
_sym_db.RegisterMessage(UnionType)

ListType = _reflection.GeneratedProtocolMessageType('ListType', (_message.Message,), {
  'DESCRIPTOR' : _LISTTYPE,
  '__module__' : 'model_pb2'
  # @@protoc_insertion_point(class_scope:kclvm.doc.model.ListType)
  })
_sym_db.RegisterMessage(ListType)

DictType = _reflection.GeneratedProtocolMessageType('DictType', (_message.Message,), {
  'DESCRIPTOR' : _DICTTYPE,
  '__module__' : 'model_pb2'
  # @@protoc_insertion_point(class_scope:kclvm.doc.model.DictType)
  })
_sym_db.RegisterMessage(DictType)

SchemaType = _reflection.GeneratedProtocolMessageType('SchemaType', (_message.Message,), {
  'DESCRIPTOR' : _SCHEMATYPE,
  '__module__' : 'model_pb2'
  # @@protoc_insertion_point(class_scope:kclvm.doc.model.SchemaType)
  })
_sym_db.RegisterMessage(SchemaType)

LitType = _reflection.GeneratedProtocolMessageType('LitType', (_message.Message,), {
  'DESCRIPTOR' : _LITTYPE,
  '__module__' : 'model_pb2'
  # @@protoc_insertion_point(class_scope:kclvm.doc.model.LitType)
  })
_sym_db.RegisterMessage(LitType)


# @@protoc_insertion_point(module_scope)



# Copyright 2021 The KCL Authors. All rights reserved.

import os
import typing
import sys
import subprocess
import pathlib
import json
import glob
import hashlib
import platform
import shutil
import filelock

import kclvm.config
import kclvm.kcl.ast as ast
import kclvm.api.object as objpkg
import kclvm.compiler.build.compiler as compiler
import kclvm.compiler.vfs as vfs
import kclvm.kcl.error as kcl_error
import kclvm.api.version as ver_pkg


CLANG = "clang"
LL_FILE_PATTERN = "*.ll"
BC_FILE_PATTERN = "*.bc"
LL_FILE_SUFFIX = ".ll"
BC_FILE_SUFFIX = ".bc"
LOCK_FILE_SUFFIX = ".lock"
KCLVM_PANIC_INFO_KEY = "__kcl_PanicInfo__"
NATIVE_CACHE_DIR = ".kclvm/native_cache"
KCLVM_CLI_SUB_CMD = "build"
CACHE_OPTION = vfs.CacheOption(cache_dir=NATIVE_CACHE_DIR)


def native_run(
    path_list: typing.List[str], *, ast_prog: ast.Program
) -> objpkg.KCLResult:

    from kclvm.internal.kclx.transformer import transform_ast_to_kclx_ast_json_str

    # Config

    _no_link = True
    _exe = ".exe" if os.name == "nt" else ""
    _executable_root = os.path.dirname(os.path.dirname(sys.executable))
    _kclvm_cli = f"{_executable_root}/bin/kclvm-cli{_exe}"
    _clang = f"{_executable_root}/tools/clang/bin/{CLANG}{_exe}"
    _clang = _clang if os.path.exists(_clang) else f"{CLANG}{_exe}"
    _rust_libstd_name = (
        pathlib.Path(f"{_executable_root}/lib/rust-libstd-name.txt").read_text().strip()
    )
    _rust_libstd_dylib = f"{_executable_root}/lib/{_rust_libstd_name}"
    _kclvm_bc = f"{_executable_root}/include/_kclvm.bc"
    _a_out_ast_json = "_a.out.ast.json"
    _a_out_bc = "_a.out.ll"
    _lib_suffix = get_dylib_suffix()
    _a_out_dylib = f"_a.out.{_lib_suffix}"
    _out_bc_files = []

    # Resolve Program
    compiler.CompileProgram(ast_prog)
    # Build Program with kclvm-cli, clang with cache
    root = ast_prog.root
    modfile = vfs.LoadModFile(root)
    enable_cache = modfile.build.enable_pkg_cache
    if enable_cache:
        build_paths = []
        check_sum = ast_prog.get_check_sum(root)
        cache_dir = f"{root}/{NATIVE_CACHE_DIR}/{ver_pkg.VERSION}-{ver_pkg.CHECKSUM}"
        pathlib.Path(cache_dir).mkdir(parents=True, exist_ok=True)
        for pkgpath in ast_prog.pkgs:
            is_main_pkg = pkgpath == ast.Program.MAIN_PKGPATH
            compile_prog = ast.Program(
                root=root, main=ast_prog.main, pkgs={pkgpath: ast_prog.pkgs[pkgpath]}
            )
            if is_main_pkg:
                check_sum = compile_prog.get_check_sum(root)
                main_virtual_filename = f"{check_sum}.k"
            else:
                main_virtual_filename = ""
            bc_path = (
                f"{cache_dir}/{pkgpath}"
                if not is_main_pkg
                else f"{cache_dir}/{check_sum}"
            )
            with filelock.FileLock(bc_path + LOCK_FILE_SUFFIX):
                dylib_relative_path = (
                    vfs.LoadMainPkgCache(
                        root, main_virtual_filename, option=CACHE_OPTION
                    )
                    if is_main_pkg
                    else vfs.LoadPkgCache(root, pkgpath, option=CACHE_OPTION)
                )
                # If AST module has been modified, ignore the dylib cache
                if ast_prog.cmd_overrides and is_main_pkg:
                    dylib_relative_path = None
                if dylib_relative_path is None:
                    # Build dylib
                    ast_json = transform_ast_to_kclx_ast_json_str(compile_prog)
                    with open(_a_out_ast_json, "w") as file:
                        file.write(ast_json)
                    dylib_path = (
                        f"{cache_dir}/{pkgpath}.{_lib_suffix}"
                        if not is_main_pkg
                        else f"{cache_dir}/{check_sum}.{_lib_suffix}"
                    )
                    if kclvm.config.verbose > 3:
                        print(f"Compiling {pkgpath}")
                    if os.path.exists(bc_path):
                        os.remove(bc_path)
                    process = subprocess.run(
                        [
                            _kclvm_cli,
                            KCLVM_CLI_SUB_CMD,
                            _a_out_ast_json,
                            "--bc",
                            _kclvm_bc,
                            "-o",
                            bc_path,
                            "--linkmode",
                            "no_link",
                        ]
                    )
                    if process.returncode != 0:
                        raise Exception(
                            f"stdout: {process.stdout}, stderr: {process.stderr}"
                        )
                    process = subprocess.run(
                        [
                            _clang,
                            "-Wno-override-module",
                            "-Wno-error=unused-command-line-argument",
                            "-Wno-unused-command-line-argument",
                            "-shared",
                            "-undefined",
                            "dynamic_lookup",
                            f"-Wl,-rpath,{_executable_root}/lib",
                            f"-L{_executable_root}/lib",
                            "-lkclvm_native_shared",
                            f"-I{_executable_root}/include",
                            bc_path + LL_FILE_SUFFIX,
                            _rust_libstd_dylib,
                            "-fPIC",
                            "-o",
                            dylib_path,
                        ]
                    )
                    if process.returncode != 0:
                        raise Exception(
                            f"stdout: {process.stdout}, stderr: {process.stderr}"
                        )
                    dylib_relative_path = dylib_path.replace(root, ".", 1)
                    if not is_main_pkg:
                        vfs.SavePkgCache(
                            root, pkgpath, dylib_relative_path, option=CACHE_OPTION
                        )
                    else:
                        vfs.SaveMainPkgCache(
                            root,
                            main_virtual_filename,
                            dylib_relative_path,
                            option=CACHE_OPTION,
                        )
                    _out_bc_files.append(bc_path + LL_FILE_SUFFIX)
                else:
                    if dylib_relative_path.startswith("."):
                        dylib_path = dylib_relative_path.replace(".", root, 1)
            build_paths.append(dylib_path)
        _a_out_dylib = f"{cache_dir}/{check_sum}.out.{_lib_suffix}"

        process = subprocess.run(
            [
                _clang,
                "-Wno-override-module",
                "-Wno-error=unused-command-line-argument",
                "-Wno-unused-command-line-argument",
                "-shared",
                "-undefined",
                "dynamic_lookup",
                f"-Wl,-rpath,{_executable_root}/lib",
                f"-L{_executable_root}/lib",
                "-lkclvm_native_shared",
                f"-I{_executable_root}/include",
                *build_paths,
                _rust_libstd_dylib,
                "-fPIC",
                "-o",
                _a_out_dylib,
            ]
        )
        if process.returncode != 0:
            raise Exception(f"stdout: {process.stdout}, stderr: {process.stderr}")
    else:
        # Transfrom Program
        ast_json = transform_ast_to_kclx_ast_json_str(ast_prog)
        with open(_a_out_ast_json, "w") as file:
            file.write(ast_json)
        if _no_link and os.path.exists(_a_out_bc):
            os.remove(_a_out_bc)
        if _no_link:
            _out_bc_files = glob.glob(_a_out_bc + LL_FILE_PATTERN)
            for file in _out_bc_files:
                if os.path.exists(file):
                    os.remove(file)
        # kclvm compile
        process = subprocess.run(
            [
                _kclvm_cli,
                KCLVM_CLI_SUB_CMD,
                _a_out_ast_json,
                "--bc",
                _kclvm_bc,
                "-o",
                _a_out_bc,
                *(["--linkmode", "no_link"] if _no_link else []),
            ]
        )
        if process.returncode != 0:
            raise Exception(f"stdout: {process.stdout}, stderr: {process.stderr}")
        _out_bc_files = glob.glob(_a_out_bc + LL_FILE_PATTERN)
        # clang
        if _no_link:
            process = subprocess.run(
                [
                    _clang,
                    "-Wno-override-module",
                    "-Wno-error=unused-command-line-argument",
                    "-Wno-unused-command-line-argument",
                    "-shared",
                    "-undefined",
                    "dynamic_lookup",
                    f"-Wl,-rpath,{_executable_root}/lib",
                    f"-L{_executable_root}/lib",
                    "-lkclvm_native",
                    f"-I{_executable_root}/include",
                    *_out_bc_files,
                    _rust_libstd_dylib,
                    f"{_executable_root}/lib/libkclvm.{_lib_suffix}",
                    "-fno-lto",
                    "-fPIC",
                    "-o",
                    _a_out_dylib,
                ]
            )
        else:
            process = subprocess.run(
                [
                    _clang,
                    "-Wno-override-module",
                    "-Wno-error=unused-command-line-argument",
                    "-Wno-unused-command-line-argument",
                    "-shared",
                    "-undefined",
                    "dynamic_lookup",
                    f"-Wl,-rpath,{_executable_root}/lib",
                    f"-L{_executable_root}/lib",
                    "-lkclvm_native",
                    f"-I{_executable_root}/include",
                    _a_out_bc,
                    _rust_libstd_dylib,
                    f"{_executable_root}/lib/libkclvm.{_lib_suffix}",
                    "-fno-lto",
                    "-fPIC",
                    "-o",
                    _a_out_dylib,
                ]
            )
        if process.returncode != 0:
            raise Exception(f"stdout: {process.stdout}, stderr: {process.stderr}")

    # run app
    result = native_run_dylib(path_list, _a_out_dylib)

    if not kclvm.config.debug:
        if os.path.exists(_a_out_ast_json):
            os.remove(_a_out_ast_json)
        if not enable_cache:
            if _no_link:
                for file in _out_bc_files:
                    if os.path.exists(file):
                        os.remove(file)
            else:
                if os.path.exists(_a_out_bc):
                    os.remove(_a_out_bc)
            if os.path.exists(_a_out_dylib):
                os.remove(_a_out_dylib)

    return result


def native_run_dylib(
    path_list: typing.List[str], dylib_path: str, should_exit: bool = False
) -> objpkg.KCLResult:
    """Native run with dylib"""
    import kclvm_plugin as kclvm_plugin

    # run app
    ctx = kclvm_plugin.AppContext(os.path.abspath(dylib_path))

    # init options
    ctx.InitOptions(kclvm.config.arguments)

    # run app
    json_result = ctx.RunApp(
        strict_range_check=kclvm.config.strict_range_check,
        disable_none=kclvm.config.disable_none,
        disable_schema_check=kclvm.config.disable_schema_check,
        list_option_mode=kclvm.config.list_option_mode,
        debug_mode=kclvm.config.debug,
    )
    warn_json_result = ctx.GetWarn()

    if kclvm.config.list_option_mode:
        print(json_result, end="")
        return objpkg.KCLResult({})

    try:
        data = json.loads(json_result)
    except Exception as e:
        raise Exception(f"Exception={e}, json_result={json_result}")

    panic_info = {}
    if KCLVM_PANIC_INFO_KEY in data:
        panic_info = data
    else:
        if warn_json_result:
            try:
                panic_info = json.loads(warn_json_result)
            except Exception as e:
                raise Exception(f"Exception={e}, warn_json_result={warn_json_result}")
        else:
            panic_info = {}

    # check panic_info
    if panic_info.get(KCLVM_PANIC_INFO_KEY):
        err_type_code = panic_info["err_type_code"]
        if err_type_code:
            err_type = kcl_error.ErrType((err_type_code,))
        else:
            err_type = kcl_error.ErrType.EvaluationError_TYPE

        file_msg = [
            kcl_error.ErrFileMsg(
                filename=panic_info.get("kcl_file"),
                line_no=panic_info.get("kcl_line"),
                col_no=panic_info.get("kcl_col"),
                arg_msg=panic_info.get("kcl_arg_msg"),
            )
        ]
        if kclvm.config.debug and kclvm.config.verbose >= 2:
            rust_filename = panic_info.get("rust_file")
            rust_line = panic_info.get("rust_line")
            rust_col = panic_info.get("rust_col")
            print(f"Rust error info: {rust_filename}:{rust_line}:{rust_col}")

        config_meta_file_msg = kcl_error.ErrFileMsg(
            filename=panic_info.get("kcl_config_meta_file"),
            line_no=panic_info.get("kcl_config_meta_line"),
            col_no=panic_info.get("kcl_config_meta_col"),
            arg_msg=panic_info.get("kcl_config_meta_arg_msg"),
        )
        if config_meta_file_msg.arg_msg:
            file_msg.append(config_meta_file_msg)

        if panic_info.get("is_warning") or panic_info.get("is_warnning"):
            kcl_error.report_warning(
                err_type=err_type, file_msgs=[], arg_msg=panic_info.get("message")
            )
        else:
            kcl_error.report_exception(
                err_type=err_type,
                file_msgs=file_msg,
                arg_msg=panic_info.get("message"),
            )

    return objpkg.KCLResult(data, os.path.abspath(path_list[-1]))


def native_try_run_dylib(
    root, path_list: typing.List[str], dylib_path: str
) -> typing.Optional[objpkg.KCLResult]:
    if os.path.exists(dylib_path):
        if is_linux_platform():
            # Run ldd
            _ldd = "ldd"
            _so_not_found_flag = " => not found"
            try:
                process = subprocess.run(
                    [
                        _ldd,
                        dylib_path,
                    ],
                    capture_output=True,
                )
                if process.returncode != 0:
                    return None
                linked_so_path_list = [
                    p.strip("\t ")
                    for p in str(process.stdout, encoding="utf-8").split("\n")
                    if p
                ]
                not_found_so_path_list = [
                    p.replace(_so_not_found_flag, "").strip()
                    for p in linked_so_path_list
                    if _so_not_found_flag in p
                ]
                target_path = not_found_so_path_list[-1].rsplit("/", 1)[0]
                source_path = (
                    f"{root}/{NATIVE_CACHE_DIR}/{ver_pkg.VERSION}-{ver_pkg.CHECKSUM}"
                )

                if os.path.exists(target_path):
                    shutil.rmtree(target_path)
                os.makedirs(target_path)
                if os.path.exists(source_path):
                    for root, dirs, files in os.walk(source_path):
                        for file in files:
                            src_file = os.path.join(root, file)
                            shutil.copy(src_file, target_path)

                return native_run_dylib(path_list, dylib_path, should_exit=True)
            except Exception:
                return None
    return None


def get_path_list_dylib_path(root: str, path_list: typing.List[str]) -> str:
    check_sum = hashlib.md5()
    if not path_list or not root:
        return ""
    for filename in path_list:
        if os.path.isfile(filename):
            filename = os.path.abspath(filename)
            check_sum.update(
                (filename.replace(root, ".", 1) if root else filename).encode(
                    encoding="utf-8"
                )
            )
            with open(filename, "rb") as f:
                check_sum.update(f.read())
    cache_dir = f"{root}/{NATIVE_CACHE_DIR}/{ver_pkg.VERSION}-{ver_pkg.CHECKSUM}"
    dylib_path = "{}/{}.out.{}".format(
        cache_dir, check_sum.hexdigest(), get_dylib_suffix()
    )
    return dylib_path


def get_dylib_suffix() -> str:
    """Get dylib suffix on diffrent platform"""
    sysstr = platform.system()
    if sysstr == "Windows":
        lib_suffix = "dll"
    elif sysstr == "Linux":
        lib_suffix = "so"
    else:
        lib_suffix = "dylib"
    return lib_suffix


def is_linux_platform() -> bool:
    """Platform is linux"""
    return platform.system() == "Linux"



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: gpyrpc/gpyrpc.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x13gpyrpc/gpyrpc.proto\x12\x06gpyrpc\x1a\x19google/protobuf/any.proto\x1a google/protobuf/descriptor.proto\")\n\nCmdArgSpec\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"[\n\x0f\x43mdOverrideSpec\x12\x0f\n\x07pkgpath\x18\x01 \x01(\t\x12\x12\n\nfield_path\x18\x02 \x01(\t\x12\x13\n\x0b\x66ield_value\x18\x03 \x01(\t\x12\x0e\n\x06\x61\x63tion\x18\x04 \x01(\t\"f\n\x0cRestResponse\x12$\n\x06result\x18\x01 \x01(\x0b\x32\x14.google.protobuf.Any\x12\r\n\x05\x65rror\x18\x02 \x01(\t\x12!\n\x07kcl_err\x18\x03 \x01(\x0b\x32\x10.gpyrpc.KclError\"`\n\x08KclError\x12\x0e\n\x06\x65wcode\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0b\n\x03msg\x18\x03 \x01(\t\x12)\n\x0b\x65rror_infos\x18\x04 \x03(\x0b\x32\x14.gpyrpc.KclErrorInfo\"w\n\x0cKclErrorInfo\x12\x11\n\terr_level\x18\x01 \x01(\t\x12\x0f\n\x07\x61rg_msg\x18\x02 \x01(\t\x12\x10\n\x08\x66ilename\x18\x03 \x01(\t\x12\x10\n\x08src_code\x18\x04 \x01(\t\x12\x0f\n\x07line_no\x18\x05 \x01(\t\x12\x0e\n\x06\x63ol_no\x18\x06 \x01(\t\"\x1a\n\tPing_Args\x12\r\n\x05value\x18\x01 \x01(\t\"\x1c\n\x0bPing_Result\x12\r\n\x05value\x18\x01 \x01(\t\"\x11\n\x0fListMethod_Args\"-\n\x11ListMethod_Result\x12\x18\n\x10method_name_list\x18\x01 \x03(\t\"Z\n\x17ParseFile_LarkTree_Args\x12\x10\n\x08\x66ilename\x18\x01 \x01(\t\x12\x13\n\x0bsource_code\x18\x02 \x01(\t\x12\x18\n\x10ignore_file_line\x18\x03 \x01(\x08\"3\n\x19ParseFile_LarkTree_Result\x12\x16\n\x0elark_tree_json\x18\x01 \x01(\t\";\n\x12ParseFile_AST_Args\x12\x10\n\x08\x66ilename\x18\x01 \x01(\t\x12\x13\n\x0bsource_code\x18\x02 \x01(\t\"(\n\x14ParseFile_AST_Result\x12\x10\n\x08\x61st_json\x18\x01 \x01(\t\"0\n\x15ParseProgram_AST_Args\x12\x17\n\x0fk_filename_list\x18\x01 \x03(\t\"+\n\x17ParseProgram_AST_Result\x12\x10\n\x08\x61st_json\x18\x01 \x01(\t\"\xe0\x02\n\x10\x45xecProgram_Args\x12\x10\n\x08work_dir\x18\x01 \x01(\t\x12\x17\n\x0fk_filename_list\x18\x02 \x03(\t\x12\x13\n\x0bk_code_list\x18\x03 \x03(\t\x12 \n\x04\x61rgs\x18\x04 \x03(\x0b\x32\x12.gpyrpc.CmdArgSpec\x12*\n\toverrides\x18\x05 \x03(\x0b\x32\x17.gpyrpc.CmdOverrideSpec\x12\x1b\n\x13\x64isable_yaml_result\x18\x06 \x01(\x08\x12\x1a\n\x12print_override_ast\x18\x07 \x01(\x08\x12\x1a\n\x12strict_range_check\x18\x08 \x01(\x08\x12\x14\n\x0c\x64isable_none\x18\t \x01(\x08\x12\x0f\n\x07verbose\x18\n \x01(\x05\x12\r\n\x05\x64\x65\x62ug\x18\x0b \x01(\x05\x12\x11\n\tsort_keys\x18\x0c \x01(\x08\x12 \n\x18include_schema_type_path\x18\r \x01(\x08\"T\n\x12\x45xecProgram_Result\x12\x13\n\x0bjson_result\x18\x01 \x01(\t\x12\x13\n\x0byaml_result\x18\x02 \x01(\t\x12\x14\n\x0c\x65scaped_time\x18\x65 \x01(\t\"\'\n\x10ResetPlugin_Args\x12\x13\n\x0bplugin_root\x18\x01 \x01(\t\"\x14\n\x12ResetPlugin_Result\"!\n\x0f\x46ormatCode_Args\x12\x0e\n\x06source\x18\x01 \x01(\t\"&\n\x11\x46ormatCode_Result\x12\x11\n\tformatted\x18\x01 \x01(\x0c\"\x1f\n\x0f\x46ormatPath_Args\x12\x0c\n\x04path\x18\x01 \x01(\t\")\n\x11\x46ormatPath_Result\x12\x14\n\x0c\x63hangedPaths\x18\x01 \x03(\t\"\x1d\n\rLintPath_Args\x12\x0c\n\x04path\x18\x01 \x01(\t\"\"\n\x0fLintPath_Result\x12\x0f\n\x07results\x18\x01 \x03(\t\"F\n\x11OverrideFile_Args\x12\x0c\n\x04\x66ile\x18\x01 \x01(\t\x12\r\n\x05specs\x18\x02 \x03(\t\x12\x14\n\x0cimport_paths\x18\x03 \x03(\t\"%\n\x13OverrideFile_Result\x12\x0e\n\x06result\x18\x01 \x01(\x08\"\x1d\n\rEvalCode_Args\x12\x0c\n\x04\x63ode\x18\x01 \x01(\t\"&\n\x0f\x45valCode_Result\x12\x13\n\x0bjson_result\x18\x02 \x01(\t\" \n\x10ResolveCode_Args\x12\x0c\n\x04\x63ode\x18\x01 \x01(\t\"%\n\x12ResolveCode_Result\x12\x0f\n\x07success\x18\x01 \x01(\x08\"E\n\x12GetSchemaType_Args\x12\x0c\n\x04\x66ile\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12\x13\n\x0bschema_name\x18\x03 \x01(\t\"A\n\x14GetSchemaType_Result\x12)\n\x10schema_type_list\x18\x01 \x03(\x0b\x32\x0f.gpyrpc.KclType\"g\n\x11ValidateCode_Args\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12\x0e\n\x06schema\x18\x03 \x01(\t\x12\x16\n\x0e\x61ttribute_name\x18\x04 \x01(\t\x12\x0e\n\x06\x66ormat\x18\x05 \x01(\t\";\n\x13ValidateCode_Result\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x13\n\x0b\x65rr_message\x18\x02 \x01(\t\"+\n\x0b\x43odeSnippet\x12\x0e\n\x06schema\x18\x01 \x01(\t\x12\x0c\n\x04rule\x18\x02 \x01(\t\"<\n\x0fSpliceCode_Args\x12)\n\x0c\x63odeSnippets\x18\x01 \x03(\x0b\x32\x13.gpyrpc.CodeSnippet\"\'\n\x11SpliceCode_Result\x12\x12\n\nspliceCode\x18\x01 \x01(\t\":\n\x08Position\x12\x0c\n\x04line\x18\x01 \x01(\x03\x12\x0e\n\x06\x63olumn\x18\x02 \x01(\x03\x12\x10\n\x08\x66ilename\x18\x03 \x01(\t\"J\n\rComplete_Args\x12\x1d\n\x03pos\x18\x01 \x01(\x0b\x32\x10.gpyrpc.Position\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0c\n\x04\x63ode\x18\x03 \x01(\t\"(\n\x0f\x43omplete_Result\x12\x15\n\rcompleteItems\x18\x01 \x01(\t\";\n\x0cGoToDef_Args\x12\x1d\n\x03pos\x18\x01 \x01(\x0b\x32\x10.gpyrpc.Position\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\"#\n\x0eGoToDef_Result\x12\x11\n\tlocations\x18\x01 \x01(\t\"1\n\x13\x44ocumentSymbol_Args\x12\x0c\n\x04\x66ile\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\"\'\n\x15\x44ocumentSymbol_Result\x12\x0e\n\x06symbol\x18\x01 \x01(\t\"9\n\nHover_Args\x12\x1d\n\x03pos\x18\x01 \x01(\x0b\x32\x10.gpyrpc.Position\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\"#\n\x0cHover_Result\x12\x13\n\x0bhoverResult\x18\x01 \x01(\t\"i\n\x11ListDepFiles_Args\x12\x10\n\x08work_dir\x18\x01 \x01(\t\x12\x14\n\x0cuse_abs_path\x18\x02 \x01(\x08\x12\x13\n\x0binclude_all\x18\x03 \x01(\x08\x12\x17\n\x0fuse_fast_parser\x18\x04 \x01(\x08\"F\n\x13ListDepFiles_Result\x12\x0f\n\x07pkgroot\x18\x01 \x01(\t\x12\x0f\n\x07pkgpath\x18\x02 \x01(\t\x12\r\n\x05\x66iles\x18\x03 \x03(\t\"9\n\x16LoadSettingsFiles_Args\x12\x10\n\x08work_dir\x18\x01 \x01(\t\x12\r\n\x05\x66iles\x18\x02 \x03(\t\"q\n\x18LoadSettingsFiles_Result\x12*\n\x0fkcl_cli_configs\x18\x01 \x01(\x0b\x32\x11.gpyrpc.CliConfig\x12)\n\x0bkcl_options\x18\x02 \x03(\x0b\x32\x14.gpyrpc.KeyValuePair\"\xa6\x01\n\tCliConfig\x12\r\n\x05\x66iles\x18\x01 \x03(\t\x12\x0e\n\x06output\x18\x02 \x01(\t\x12\x11\n\toverrides\x18\x03 \x03(\t\x12\x15\n\rpath_selector\x18\x04 \x03(\t\x12\x1a\n\x12strict_range_check\x18\x05 \x01(\x08\x12\x14\n\x0c\x64isable_none\x18\x06 \x01(\x08\x12\x0f\n\x07verbose\x18\x07 \x01(\x03\x12\r\n\x05\x64\x65\x62ug\x18\x08 \x01(\x08\"*\n\x0cKeyValuePair\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"\xf4\x02\n\x07KclType\x12\x0c\n\x04type\x18\x01 \x01(\t\x12$\n\x0bunion_types\x18\x02 \x03(\x0b\x32\x0f.gpyrpc.KclType\x12\x0f\n\x07\x64\x65\x66\x61ult\x18\x03 \x01(\t\x12\x13\n\x0bschema_name\x18\x04 \x01(\t\x12\x12\n\nschema_doc\x18\x05 \x01(\t\x12\x33\n\nproperties\x18\x06 \x03(\x0b\x32\x1f.gpyrpc.KclType.PropertiesEntry\x12\x10\n\x08required\x18\x07 \x03(\t\x12\x1c\n\x03key\x18\x08 \x01(\x0b\x32\x0f.gpyrpc.KclType\x12\x1d\n\x04item\x18\t \x01(\x0b\x32\x0f.gpyrpc.KclType\x12\x0c\n\x04line\x18\n \x01(\x05\x12%\n\ndecorators\x18\x0b \x03(\x0b\x32\x11.gpyrpc.Decorator\x1a\x42\n\x0fPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x1e\n\x05value\x18\x02 \x01(\x0b\x32\x0f.gpyrpc.KclType:\x02\x38\x01\"\x90\x01\n\tDecorator\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\targuments\x18\x02 \x03(\t\x12\x31\n\x08keywords\x18\x03 \x03(\x0b\x32\x1f.gpyrpc.Decorator.KeywordsEntry\x1a/\n\rKeywordsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x32\x82\x01\n\x0e\x42uiltinService\x12.\n\x04Ping\x12\x11.gpyrpc.Ping_Args\x1a\x13.gpyrpc.Ping_Result\x12@\n\nListMethod\x12\x17.gpyrpc.ListMethod_Args\x1a\x19.gpyrpc.ListMethod_Result2\xb4\x0b\n\x0cKclvmService\x12.\n\x04Ping\x12\x11.gpyrpc.Ping_Args\x1a\x13.gpyrpc.Ping_Result\x12X\n\x12ParseFile_LarkTree\x12\x1f.gpyrpc.ParseFile_LarkTree_Args\x1a!.gpyrpc.ParseFile_LarkTree_Result\x12I\n\rParseFile_AST\x12\x1a.gpyrpc.ParseFile_AST_Args\x1a\x1c.gpyrpc.ParseFile_AST_Result\x12R\n\x10ParseProgram_AST\x12\x1d.gpyrpc.ParseProgram_AST_Args\x1a\x1f.gpyrpc.ParseProgram_AST_Result\x12\x43\n\x0b\x45xecProgram\x12\x18.gpyrpc.ExecProgram_Args\x1a\x1a.gpyrpc.ExecProgram_Result\x12\x43\n\x0bResetPlugin\x12\x18.gpyrpc.ResetPlugin_Args\x1a\x1a.gpyrpc.ResetPlugin_Result\x12@\n\nFormatCode\x12\x17.gpyrpc.FormatCode_Args\x1a\x19.gpyrpc.FormatCode_Result\x12@\n\nFormatPath\x12\x17.gpyrpc.FormatPath_Args\x1a\x19.gpyrpc.FormatPath_Result\x12:\n\x08LintPath\x12\x15.gpyrpc.LintPath_Args\x1a\x17.gpyrpc.LintPath_Result\x12\x46\n\x0cOverrideFile\x12\x19.gpyrpc.OverrideFile_Args\x1a\x1b.gpyrpc.OverrideFile_Result\x12:\n\x08\x45valCode\x12\x15.gpyrpc.EvalCode_Args\x1a\x17.gpyrpc.EvalCode_Result\x12\x43\n\x0bResolveCode\x12\x18.gpyrpc.ResolveCode_Args\x1a\x1a.gpyrpc.ResolveCode_Result\x12I\n\rGetSchemaType\x12\x1a.gpyrpc.GetSchemaType_Args\x1a\x1c.gpyrpc.GetSchemaType_Result\x12\x46\n\x0cValidateCode\x12\x19.gpyrpc.ValidateCode_Args\x1a\x1b.gpyrpc.ValidateCode_Result\x12@\n\nSpliceCode\x12\x17.gpyrpc.SpliceCode_Args\x1a\x19.gpyrpc.SpliceCode_Result\x12:\n\x08\x43omplete\x12\x15.gpyrpc.Complete_Args\x1a\x17.gpyrpc.Complete_Result\x12\x37\n\x07GoToDef\x12\x14.gpyrpc.GoToDef_Args\x1a\x16.gpyrpc.GoToDef_Result\x12L\n\x0e\x44ocumentSymbol\x12\x1b.gpyrpc.DocumentSymbol_Args\x1a\x1d.gpyrpc.DocumentSymbol_Result\x12\x31\n\x05Hover\x12\x12.gpyrpc.Hover_Args\x1a\x14.gpyrpc.Hover_Result\x12\x46\n\x0cListDepFiles\x12\x19.gpyrpc.ListDepFiles_Args\x1a\x1b.gpyrpc.ListDepFiles_Result\x12U\n\x11LoadSettingsFiles\x12\x1e.gpyrpc.LoadSettingsFiles_Args\x1a .gpyrpc.LoadSettingsFiles_ResultB0Z.kusionstack.io/kclvm-go/pkg/spec/gpyrpc;gpyrpcb\x06proto3')



_CMDARGSPEC = DESCRIPTOR.message_types_by_name['CmdArgSpec']
_CMDOVERRIDESPEC = DESCRIPTOR.message_types_by_name['CmdOverrideSpec']
_RESTRESPONSE = DESCRIPTOR.message_types_by_name['RestResponse']
_KCLERROR = DESCRIPTOR.message_types_by_name['KclError']
_KCLERRORINFO = DESCRIPTOR.message_types_by_name['KclErrorInfo']
_PING_ARGS = DESCRIPTOR.message_types_by_name['Ping_Args']
_PING_RESULT = DESCRIPTOR.message_types_by_name['Ping_Result']
_LISTMETHOD_ARGS = DESCRIPTOR.message_types_by_name['ListMethod_Args']
_LISTMETHOD_RESULT = DESCRIPTOR.message_types_by_name['ListMethod_Result']
_PARSEFILE_LARKTREE_ARGS = DESCRIPTOR.message_types_by_name['ParseFile_LarkTree_Args']
_PARSEFILE_LARKTREE_RESULT = DESCRIPTOR.message_types_by_name['ParseFile_LarkTree_Result']
_PARSEFILE_AST_ARGS = DESCRIPTOR.message_types_by_name['ParseFile_AST_Args']
_PARSEFILE_AST_RESULT = DESCRIPTOR.message_types_by_name['ParseFile_AST_Result']
_PARSEPROGRAM_AST_ARGS = DESCRIPTOR.message_types_by_name['ParseProgram_AST_Args']
_PARSEPROGRAM_AST_RESULT = DESCRIPTOR.message_types_by_name['ParseProgram_AST_Result']
_EXECPROGRAM_ARGS = DESCRIPTOR.message_types_by_name['ExecProgram_Args']
_EXECPROGRAM_RESULT = DESCRIPTOR.message_types_by_name['ExecProgram_Result']
_RESETPLUGIN_ARGS = DESCRIPTOR.message_types_by_name['ResetPlugin_Args']
_RESETPLUGIN_RESULT = DESCRIPTOR.message_types_by_name['ResetPlugin_Result']
_FORMATCODE_ARGS = DESCRIPTOR.message_types_by_name['FormatCode_Args']
_FORMATCODE_RESULT = DESCRIPTOR.message_types_by_name['FormatCode_Result']
_FORMATPATH_ARGS = DESCRIPTOR.message_types_by_name['FormatPath_Args']
_FORMATPATH_RESULT = DESCRIPTOR.message_types_by_name['FormatPath_Result']
_LINTPATH_ARGS = DESCRIPTOR.message_types_by_name['LintPath_Args']
_LINTPATH_RESULT = DESCRIPTOR.message_types_by_name['LintPath_Result']
_OVERRIDEFILE_ARGS = DESCRIPTOR.message_types_by_name['OverrideFile_Args']
_OVERRIDEFILE_RESULT = DESCRIPTOR.message_types_by_name['OverrideFile_Result']
_EVALCODE_ARGS = DESCRIPTOR.message_types_by_name['EvalCode_Args']
_EVALCODE_RESULT = DESCRIPTOR.message_types_by_name['EvalCode_Result']
_RESOLVECODE_ARGS = DESCRIPTOR.message_types_by_name['ResolveCode_Args']
_RESOLVECODE_RESULT = DESCRIPTOR.message_types_by_name['ResolveCode_Result']
_GETSCHEMATYPE_ARGS = DESCRIPTOR.message_types_by_name['GetSchemaType_Args']
_GETSCHEMATYPE_RESULT = DESCRIPTOR.message_types_by_name['GetSchemaType_Result']
_VALIDATECODE_ARGS = DESCRIPTOR.message_types_by_name['ValidateCode_Args']
_VALIDATECODE_RESULT = DESCRIPTOR.message_types_by_name['ValidateCode_Result']
_CODESNIPPET = DESCRIPTOR.message_types_by_name['CodeSnippet']
_SPLICECODE_ARGS = DESCRIPTOR.message_types_by_name['SpliceCode_Args']
_SPLICECODE_RESULT = DESCRIPTOR.message_types_by_name['SpliceCode_Result']
_POSITION = DESCRIPTOR.message_types_by_name['Position']
_COMPLETE_ARGS = DESCRIPTOR.message_types_by_name['Complete_Args']
_COMPLETE_RESULT = DESCRIPTOR.message_types_by_name['Complete_Result']
_GOTODEF_ARGS = DESCRIPTOR.message_types_by_name['GoToDef_Args']
_GOTODEF_RESULT = DESCRIPTOR.message_types_by_name['GoToDef_Result']
_DOCUMENTSYMBOL_ARGS = DESCRIPTOR.message_types_by_name['DocumentSymbol_Args']
_DOCUMENTSYMBOL_RESULT = DESCRIPTOR.message_types_by_name['DocumentSymbol_Result']
_HOVER_ARGS = DESCRIPTOR.message_types_by_name['Hover_Args']
_HOVER_RESULT = DESCRIPTOR.message_types_by_name['Hover_Result']
_LISTDEPFILES_ARGS = DESCRIPTOR.message_types_by_name['ListDepFiles_Args']
_LISTDEPFILES_RESULT = DESCRIPTOR.message_types_by_name['ListDepFiles_Result']
_LOADSETTINGSFILES_ARGS = DESCRIPTOR.message_types_by_name['LoadSettingsFiles_Args']
_LOADSETTINGSFILES_RESULT = DESCRIPTOR.message_types_by_name['LoadSettingsFiles_Result']
_CLICONFIG = DESCRIPTOR.message_types_by_name['CliConfig']
_KEYVALUEPAIR = DESCRIPTOR.message_types_by_name['KeyValuePair']
_KCLTYPE = DESCRIPTOR.message_types_by_name['KclType']
_KCLTYPE_PROPERTIESENTRY = _KCLTYPE.nested_types_by_name['PropertiesEntry']
_DECORATOR = DESCRIPTOR.message_types_by_name['Decorator']
_DECORATOR_KEYWORDSENTRY = _DECORATOR.nested_types_by_name['KeywordsEntry']
CmdArgSpec = _reflection.GeneratedProtocolMessageType('CmdArgSpec', (_message.Message,), {
  'DESCRIPTOR' : _CMDARGSPEC,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.CmdArgSpec)
  })
_sym_db.RegisterMessage(CmdArgSpec)

CmdOverrideSpec = _reflection.GeneratedProtocolMessageType('CmdOverrideSpec', (_message.Message,), {
  'DESCRIPTOR' : _CMDOVERRIDESPEC,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.CmdOverrideSpec)
  })
_sym_db.RegisterMessage(CmdOverrideSpec)

RestResponse = _reflection.GeneratedProtocolMessageType('RestResponse', (_message.Message,), {
  'DESCRIPTOR' : _RESTRESPONSE,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.RestResponse)
  })
_sym_db.RegisterMessage(RestResponse)

KclError = _reflection.GeneratedProtocolMessageType('KclError', (_message.Message,), {
  'DESCRIPTOR' : _KCLERROR,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.KclError)
  })
_sym_db.RegisterMessage(KclError)

KclErrorInfo = _reflection.GeneratedProtocolMessageType('KclErrorInfo', (_message.Message,), {
  'DESCRIPTOR' : _KCLERRORINFO,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.KclErrorInfo)
  })
_sym_db.RegisterMessage(KclErrorInfo)

Ping_Args = _reflection.GeneratedProtocolMessageType('Ping_Args', (_message.Message,), {
  'DESCRIPTOR' : _PING_ARGS,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.Ping_Args)
  })
_sym_db.RegisterMessage(Ping_Args)

Ping_Result = _reflection.GeneratedProtocolMessageType('Ping_Result', (_message.Message,), {
  'DESCRIPTOR' : _PING_RESULT,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.Ping_Result)
  })
_sym_db.RegisterMessage(Ping_Result)

ListMethod_Args = _reflection.GeneratedProtocolMessageType('ListMethod_Args', (_message.Message,), {
  'DESCRIPTOR' : _LISTMETHOD_ARGS,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.ListMethod_Args)
  })
_sym_db.RegisterMessage(ListMethod_Args)

ListMethod_Result = _reflection.GeneratedProtocolMessageType('ListMethod_Result', (_message.Message,), {
  'DESCRIPTOR' : _LISTMETHOD_RESULT,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.ListMethod_Result)
  })
_sym_db.RegisterMessage(ListMethod_Result)

ParseFile_LarkTree_Args = _reflection.GeneratedProtocolMessageType('ParseFile_LarkTree_Args', (_message.Message,), {
  'DESCRIPTOR' : _PARSEFILE_LARKTREE_ARGS,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.ParseFile_LarkTree_Args)
  })
_sym_db.RegisterMessage(ParseFile_LarkTree_Args)

ParseFile_LarkTree_Result = _reflection.GeneratedProtocolMessageType('ParseFile_LarkTree_Result', (_message.Message,), {
  'DESCRIPTOR' : _PARSEFILE_LARKTREE_RESULT,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.ParseFile_LarkTree_Result)
  })
_sym_db.RegisterMessage(ParseFile_LarkTree_Result)

ParseFile_AST_Args = _reflection.GeneratedProtocolMessageType('ParseFile_AST_Args', (_message.Message,), {
  'DESCRIPTOR' : _PARSEFILE_AST_ARGS,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.ParseFile_AST_Args)
  })
_sym_db.RegisterMessage(ParseFile_AST_Args)

ParseFile_AST_Result = _reflection.GeneratedProtocolMessageType('ParseFile_AST_Result', (_message.Message,), {
  'DESCRIPTOR' : _PARSEFILE_AST_RESULT,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.ParseFile_AST_Result)
  })
_sym_db.RegisterMessage(ParseFile_AST_Result)

ParseProgram_AST_Args = _reflection.GeneratedProtocolMessageType('ParseProgram_AST_Args', (_message.Message,), {
  'DESCRIPTOR' : _PARSEPROGRAM_AST_ARGS,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.ParseProgram_AST_Args)
  })
_sym_db.RegisterMessage(ParseProgram_AST_Args)

ParseProgram_AST_Result = _reflection.GeneratedProtocolMessageType('ParseProgram_AST_Result', (_message.Message,), {
  'DESCRIPTOR' : _PARSEPROGRAM_AST_RESULT,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.ParseProgram_AST_Result)
  })
_sym_db.RegisterMessage(ParseProgram_AST_Result)

ExecProgram_Args = _reflection.GeneratedProtocolMessageType('ExecProgram_Args', (_message.Message,), {
  'DESCRIPTOR' : _EXECPROGRAM_ARGS,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.ExecProgram_Args)
  })
_sym_db.RegisterMessage(ExecProgram_Args)

ExecProgram_Result = _reflection.GeneratedProtocolMessageType('ExecProgram_Result', (_message.Message,), {
  'DESCRIPTOR' : _EXECPROGRAM_RESULT,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.ExecProgram_Result)
  })
_sym_db.RegisterMessage(ExecProgram_Result)

ResetPlugin_Args = _reflection.GeneratedProtocolMessageType('ResetPlugin_Args', (_message.Message,), {
  'DESCRIPTOR' : _RESETPLUGIN_ARGS,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.ResetPlugin_Args)
  })
_sym_db.RegisterMessage(ResetPlugin_Args)

ResetPlugin_Result = _reflection.GeneratedProtocolMessageType('ResetPlugin_Result', (_message.Message,), {
  'DESCRIPTOR' : _RESETPLUGIN_RESULT,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.ResetPlugin_Result)
  })
_sym_db.RegisterMessage(ResetPlugin_Result)

FormatCode_Args = _reflection.GeneratedProtocolMessageType('FormatCode_Args', (_message.Message,), {
  'DESCRIPTOR' : _FORMATCODE_ARGS,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.FormatCode_Args)
  })
_sym_db.RegisterMessage(FormatCode_Args)

FormatCode_Result = _reflection.GeneratedProtocolMessageType('FormatCode_Result', (_message.Message,), {
  'DESCRIPTOR' : _FORMATCODE_RESULT,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.FormatCode_Result)
  })
_sym_db.RegisterMessage(FormatCode_Result)

FormatPath_Args = _reflection.GeneratedProtocolMessageType('FormatPath_Args', (_message.Message,), {
  'DESCRIPTOR' : _FORMATPATH_ARGS,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.FormatPath_Args)
  })
_sym_db.RegisterMessage(FormatPath_Args)

FormatPath_Result = _reflection.GeneratedProtocolMessageType('FormatPath_Result', (_message.Message,), {
  'DESCRIPTOR' : _FORMATPATH_RESULT,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.FormatPath_Result)
  })
_sym_db.RegisterMessage(FormatPath_Result)

LintPath_Args = _reflection.GeneratedProtocolMessageType('LintPath_Args', (_message.Message,), {
  'DESCRIPTOR' : _LINTPATH_ARGS,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.LintPath_Args)
  })
_sym_db.RegisterMessage(LintPath_Args)

LintPath_Result = _reflection.GeneratedProtocolMessageType('LintPath_Result', (_message.Message,), {
  'DESCRIPTOR' : _LINTPATH_RESULT,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.LintPath_Result)
  })
_sym_db.RegisterMessage(LintPath_Result)

OverrideFile_Args = _reflection.GeneratedProtocolMessageType('OverrideFile_Args', (_message.Message,), {
  'DESCRIPTOR' : _OVERRIDEFILE_ARGS,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.OverrideFile_Args)
  })
_sym_db.RegisterMessage(OverrideFile_Args)

OverrideFile_Result = _reflection.GeneratedProtocolMessageType('OverrideFile_Result', (_message.Message,), {
  'DESCRIPTOR' : _OVERRIDEFILE_RESULT,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.OverrideFile_Result)
  })
_sym_db.RegisterMessage(OverrideFile_Result)

EvalCode_Args = _reflection.GeneratedProtocolMessageType('EvalCode_Args', (_message.Message,), {
  'DESCRIPTOR' : _EVALCODE_ARGS,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.EvalCode_Args)
  })
_sym_db.RegisterMessage(EvalCode_Args)

EvalCode_Result = _reflection.GeneratedProtocolMessageType('EvalCode_Result', (_message.Message,), {
  'DESCRIPTOR' : _EVALCODE_RESULT,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.EvalCode_Result)
  })
_sym_db.RegisterMessage(EvalCode_Result)

ResolveCode_Args = _reflection.GeneratedProtocolMessageType('ResolveCode_Args', (_message.Message,), {
  'DESCRIPTOR' : _RESOLVECODE_ARGS,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.ResolveCode_Args)
  })
_sym_db.RegisterMessage(ResolveCode_Args)

ResolveCode_Result = _reflection.GeneratedProtocolMessageType('ResolveCode_Result', (_message.Message,), {
  'DESCRIPTOR' : _RESOLVECODE_RESULT,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.ResolveCode_Result)
  })
_sym_db.RegisterMessage(ResolveCode_Result)

GetSchemaType_Args = _reflection.GeneratedProtocolMessageType('GetSchemaType_Args', (_message.Message,), {
  'DESCRIPTOR' : _GETSCHEMATYPE_ARGS,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.GetSchemaType_Args)
  })
_sym_db.RegisterMessage(GetSchemaType_Args)

GetSchemaType_Result = _reflection.GeneratedProtocolMessageType('GetSchemaType_Result', (_message.Message,), {
  'DESCRIPTOR' : _GETSCHEMATYPE_RESULT,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.GetSchemaType_Result)
  })
_sym_db.RegisterMessage(GetSchemaType_Result)

ValidateCode_Args = _reflection.GeneratedProtocolMessageType('ValidateCode_Args', (_message.Message,), {
  'DESCRIPTOR' : _VALIDATECODE_ARGS,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.ValidateCode_Args)
  })
_sym_db.RegisterMessage(ValidateCode_Args)

ValidateCode_Result = _reflection.GeneratedProtocolMessageType('ValidateCode_Result', (_message.Message,), {
  'DESCRIPTOR' : _VALIDATECODE_RESULT,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.ValidateCode_Result)
  })
_sym_db.RegisterMessage(ValidateCode_Result)

CodeSnippet = _reflection.GeneratedProtocolMessageType('CodeSnippet', (_message.Message,), {
  'DESCRIPTOR' : _CODESNIPPET,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.CodeSnippet)
  })
_sym_db.RegisterMessage(CodeSnippet)

SpliceCode_Args = _reflection.GeneratedProtocolMessageType('SpliceCode_Args', (_message.Message,), {
  'DESCRIPTOR' : _SPLICECODE_ARGS,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.SpliceCode_Args)
  })
_sym_db.RegisterMessage(SpliceCode_Args)

SpliceCode_Result = _reflection.GeneratedProtocolMessageType('SpliceCode_Result', (_message.Message,), {
  'DESCRIPTOR' : _SPLICECODE_RESULT,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.SpliceCode_Result)
  })
_sym_db.RegisterMessage(SpliceCode_Result)

Position = _reflection.GeneratedProtocolMessageType('Position', (_message.Message,), {
  'DESCRIPTOR' : _POSITION,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.Position)
  })
_sym_db.RegisterMessage(Position)

Complete_Args = _reflection.GeneratedProtocolMessageType('Complete_Args', (_message.Message,), {
  'DESCRIPTOR' : _COMPLETE_ARGS,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.Complete_Args)
  })
_sym_db.RegisterMessage(Complete_Args)

Complete_Result = _reflection.GeneratedProtocolMessageType('Complete_Result', (_message.Message,), {
  'DESCRIPTOR' : _COMPLETE_RESULT,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.Complete_Result)
  })
_sym_db.RegisterMessage(Complete_Result)

GoToDef_Args = _reflection.GeneratedProtocolMessageType('GoToDef_Args', (_message.Message,), {
  'DESCRIPTOR' : _GOTODEF_ARGS,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.GoToDef_Args)
  })
_sym_db.RegisterMessage(GoToDef_Args)

GoToDef_Result = _reflection.GeneratedProtocolMessageType('GoToDef_Result', (_message.Message,), {
  'DESCRIPTOR' : _GOTODEF_RESULT,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.GoToDef_Result)
  })
_sym_db.RegisterMessage(GoToDef_Result)

DocumentSymbol_Args = _reflection.GeneratedProtocolMessageType('DocumentSymbol_Args', (_message.Message,), {
  'DESCRIPTOR' : _DOCUMENTSYMBOL_ARGS,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.DocumentSymbol_Args)
  })
_sym_db.RegisterMessage(DocumentSymbol_Args)

DocumentSymbol_Result = _reflection.GeneratedProtocolMessageType('DocumentSymbol_Result', (_message.Message,), {
  'DESCRIPTOR' : _DOCUMENTSYMBOL_RESULT,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.DocumentSymbol_Result)
  })
_sym_db.RegisterMessage(DocumentSymbol_Result)

Hover_Args = _reflection.GeneratedProtocolMessageType('Hover_Args', (_message.Message,), {
  'DESCRIPTOR' : _HOVER_ARGS,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.Hover_Args)
  })
_sym_db.RegisterMessage(Hover_Args)

Hover_Result = _reflection.GeneratedProtocolMessageType('Hover_Result', (_message.Message,), {
  'DESCRIPTOR' : _HOVER_RESULT,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.Hover_Result)
  })
_sym_db.RegisterMessage(Hover_Result)

ListDepFiles_Args = _reflection.GeneratedProtocolMessageType('ListDepFiles_Args', (_message.Message,), {
  'DESCRIPTOR' : _LISTDEPFILES_ARGS,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.ListDepFiles_Args)
  })
_sym_db.RegisterMessage(ListDepFiles_Args)

ListDepFiles_Result = _reflection.GeneratedProtocolMessageType('ListDepFiles_Result', (_message.Message,), {
  'DESCRIPTOR' : _LISTDEPFILES_RESULT,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.ListDepFiles_Result)
  })
_sym_db.RegisterMessage(ListDepFiles_Result)

LoadSettingsFiles_Args = _reflection.GeneratedProtocolMessageType('LoadSettingsFiles_Args', (_message.Message,), {
  'DESCRIPTOR' : _LOADSETTINGSFILES_ARGS,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.LoadSettingsFiles_Args)
  })
_sym_db.RegisterMessage(LoadSettingsFiles_Args)

LoadSettingsFiles_Result = _reflection.GeneratedProtocolMessageType('LoadSettingsFiles_Result', (_message.Message,), {
  'DESCRIPTOR' : _LOADSETTINGSFILES_RESULT,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.LoadSettingsFiles_Result)
  })
_sym_db.RegisterMessage(LoadSettingsFiles_Result)

CliConfig = _reflection.GeneratedProtocolMessageType('CliConfig', (_message.Message,), {
  'DESCRIPTOR' : _CLICONFIG,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.CliConfig)
  })
_sym_db.RegisterMessage(CliConfig)

KeyValuePair = _reflection.GeneratedProtocolMessageType('KeyValuePair', (_message.Message,), {
  'DESCRIPTOR' : _KEYVALUEPAIR,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.KeyValuePair)
  })
_sym_db.RegisterMessage(KeyValuePair)

KclType = _reflection.GeneratedProtocolMessageType('KclType', (_message.Message,), {

  'PropertiesEntry' : _reflection.GeneratedProtocolMessageType('PropertiesEntry', (_message.Message,), {
    'DESCRIPTOR' : _KCLTYPE_PROPERTIESENTRY,
    '__module__' : 'gpyrpc.gpyrpc_pb2'
    # @@protoc_insertion_point(class_scope:gpyrpc.KclType.PropertiesEntry)
    })
  ,
  'DESCRIPTOR' : _KCLTYPE,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.KclType)
  })
_sym_db.RegisterMessage(KclType)
_sym_db.RegisterMessage(KclType.PropertiesEntry)

Decorator = _reflection.GeneratedProtocolMessageType('Decorator', (_message.Message,), {

  'KeywordsEntry' : _reflection.GeneratedProtocolMessageType('KeywordsEntry', (_message.Message,), {
    'DESCRIPTOR' : _DECORATOR_KEYWORDSENTRY,
    '__module__' : 'gpyrpc.gpyrpc_pb2'
    # @@protoc_insertion_point(class_scope:gpyrpc.Decorator.KeywordsEntry)
    })
  ,
  'DESCRIPTOR' : _DECORATOR,
  '__module__' : 'gpyrpc.gpyrpc_pb2'
  # @@protoc_insertion_point(class_scope:gpyrpc.Decorator)
  })
_sym_db.RegisterMessage(Decorator)
_sym_db.RegisterMessage(Decorator.KeywordsEntry)

_BUILTINSERVICE = DESCRIPTOR.services_by_name['BuiltinService']
_KCLVMSERVICE = DESCRIPTOR.services_by_name['KclvmService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z.kusionstack.io/kclvm-go/pkg/spec/gpyrpc;gpyrpc'
  _KCLTYPE_PROPERTIESENTRY._options = None
  _KCLTYPE_PROPERTIESENTRY._serialized_options = b'8\001'
  _DECORATOR_KEYWORDSENTRY._options = None
  _DECORATOR_KEYWORDSENTRY._serialized_options = b'8\001'
  _CMDARGSPEC._serialized_start=92
  _CMDARGSPEC._serialized_end=133
  _CMDOVERRIDESPEC._serialized_start=135
  _CMDOVERRIDESPEC._serialized_end=226
  _RESTRESPONSE._serialized_start=228
  _RESTRESPONSE._serialized_end=330
  _KCLERROR._serialized_start=332
  _KCLERROR._serialized_end=428
  _KCLERRORINFO._serialized_start=430
  _KCLERRORINFO._serialized_end=549
  _PING_ARGS._serialized_start=551
  _PING_ARGS._serialized_end=577
  _PING_RESULT._serialized_start=579
  _PING_RESULT._serialized_end=607
  _LISTMETHOD_ARGS._serialized_start=609
  _LISTMETHOD_ARGS._serialized_end=626
  _LISTMETHOD_RESULT._serialized_start=628
  _LISTMETHOD_RESULT._serialized_end=673
  _PARSEFILE_LARKTREE_ARGS._serialized_start=675
  _PARSEFILE_LARKTREE_ARGS._serialized_end=765
  _PARSEFILE_LARKTREE_RESULT._serialized_start=767
  _PARSEFILE_LARKTREE_RESULT._serialized_end=818
  _PARSEFILE_AST_ARGS._serialized_start=820
  _PARSEFILE_AST_ARGS._serialized_end=879
  _PARSEFILE_AST_RESULT._serialized_start=881
  _PARSEFILE_AST_RESULT._serialized_end=921
  _PARSEPROGRAM_AST_ARGS._serialized_start=923
  _PARSEPROGRAM_AST_ARGS._serialized_end=971
  _PARSEPROGRAM_AST_RESULT._serialized_start=973
  _PARSEPROGRAM_AST_RESULT._serialized_end=1016
  _EXECPROGRAM_ARGS._serialized_start=1019
  _EXECPROGRAM_ARGS._serialized_end=1371
  _EXECPROGRAM_RESULT._serialized_start=1373
  _EXECPROGRAM_RESULT._serialized_end=1457
  _RESETPLUGIN_ARGS._serialized_start=1459
  _RESETPLUGIN_ARGS._serialized_end=1498
  _RESETPLUGIN_RESULT._serialized_start=1500
  _RESETPLUGIN_RESULT._serialized_end=1520
  _FORMATCODE_ARGS._serialized_start=1522
  _FORMATCODE_ARGS._serialized_end=1555
  _FORMATCODE_RESULT._serialized_start=1557
  _FORMATCODE_RESULT._serialized_end=1595
  _FORMATPATH_ARGS._serialized_start=1597
  _FORMATPATH_ARGS._serialized_end=1628
  _FORMATPATH_RESULT._serialized_start=1630
  _FORMATPATH_RESULT._serialized_end=1671
  _LINTPATH_ARGS._serialized_start=1673
  _LINTPATH_ARGS._serialized_end=1702
  _LINTPATH_RESULT._serialized_start=1704
  _LINTPATH_RESULT._serialized_end=1738
  _OVERRIDEFILE_ARGS._serialized_start=1740
  _OVERRIDEFILE_ARGS._serialized_end=1810
  _OVERRIDEFILE_RESULT._serialized_start=1812
  _OVERRIDEFILE_RESULT._serialized_end=1849
  _EVALCODE_ARGS._serialized_start=1851
  _EVALCODE_ARGS._serialized_end=1880
  _EVALCODE_RESULT._serialized_start=1882
  _EVALCODE_RESULT._serialized_end=1920
  _RESOLVECODE_ARGS._serialized_start=1922
  _RESOLVECODE_ARGS._serialized_end=1954
  _RESOLVECODE_RESULT._serialized_start=1956
  _RESOLVECODE_RESULT._serialized_end=1993
  _GETSCHEMATYPE_ARGS._serialized_start=1995
  _GETSCHEMATYPE_ARGS._serialized_end=2064
  _GETSCHEMATYPE_RESULT._serialized_start=2066
  _GETSCHEMATYPE_RESULT._serialized_end=2131
  _VALIDATECODE_ARGS._serialized_start=2133
  _VALIDATECODE_ARGS._serialized_end=2236
  _VALIDATECODE_RESULT._serialized_start=2238
  _VALIDATECODE_RESULT._serialized_end=2297
  _CODESNIPPET._serialized_start=2299
  _CODESNIPPET._serialized_end=2342
  _SPLICECODE_ARGS._serialized_start=2344
  _SPLICECODE_ARGS._serialized_end=2404
  _SPLICECODE_RESULT._serialized_start=2406
  _SPLICECODE_RESULT._serialized_end=2445
  _POSITION._serialized_start=2447
  _POSITION._serialized_end=2505
  _COMPLETE_ARGS._serialized_start=2507
  _COMPLETE_ARGS._serialized_end=2581
  _COMPLETE_RESULT._serialized_start=2583
  _COMPLETE_RESULT._serialized_end=2623
  _GOTODEF_ARGS._serialized_start=2625
  _GOTODEF_ARGS._serialized_end=2684
  _GOTODEF_RESULT._serialized_start=2686
  _GOTODEF_RESULT._serialized_end=2721
  _DOCUMENTSYMBOL_ARGS._serialized_start=2723
  _DOCUMENTSYMBOL_ARGS._serialized_end=2772
  _DOCUMENTSYMBOL_RESULT._serialized_start=2774
  _DOCUMENTSYMBOL_RESULT._serialized_end=2813
  _HOVER_ARGS._serialized_start=2815
  _HOVER_ARGS._serialized_end=2872
  _HOVER_RESULT._serialized_start=2874
  _HOVER_RESULT._serialized_end=2909
  _LISTDEPFILES_ARGS._serialized_start=2911
  _LISTDEPFILES_ARGS._serialized_end=3016
  _LISTDEPFILES_RESULT._serialized_start=3018
  _LISTDEPFILES_RESULT._serialized_end=3088
  _LOADSETTINGSFILES_ARGS._serialized_start=3090
  _LOADSETTINGSFILES_ARGS._serialized_end=3147
  _LOADSETTINGSFILES_RESULT._serialized_start=3149
  _LOADSETTINGSFILES_RESULT._serialized_end=3262
  _CLICONFIG._serialized_start=3265
  _CLICONFIG._serialized_end=3431
  _KEYVALUEPAIR._serialized_start=3433
  _KEYVALUEPAIR._serialized_end=3475
  _KCLTYPE._serialized_start=3478
  _KCLTYPE._serialized_end=3850
  _KCLTYPE_PROPERTIESENTRY._serialized_start=3784
  _KCLTYPE_PROPERTIESENTRY._serialized_end=3850
  _DECORATOR._serialized_start=3853
  _DECORATOR._serialized_end=3997
  _DECORATOR_KEYWORDSENTRY._serialized_start=3950
  _DECORATOR_KEYWORDSENTRY._serialized_end=3997
  _BUILTINSERVICE._serialized_start=4000
  _BUILTINSERVICE._serialized_end=4130
  _KCLVMSERVICE._serialized_start=4133
  _KCLVMSERVICE._serialized_end=5593
# @@protoc_insertion_point(module_scope)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
import pathlib
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup
from pkg_resources import require
import setuptools

# Steps:
# 1. cd internal
# 2. modify kclvm version in setup.py
# 3. run `python setup.py sdist` to build package
# 4. run `twine upload dist/kclvm-0.x.x` to upload package to PyPI
# 5. input username and password of PyPI

install_requires = []
require_path = pathlib.Path(__file__).parent.joinpath("kclvm/scripts/requirements.txt")
with open(require_path) as f:
    requires = f.read().split('\n')
    for require in requires:
        install_requires.append(require)

setup(
    name='kclvm',
    author='KCL Authors',
    version='0.4.2.6',
    license='Apache License 2.0',

    description='KCLVM', 
    long_description='''A constraint-based record & functional language mainly used in configuration and policy scenarios.''',
    author_email='',  
    url='https://kusionstack.io/',  
    packages=setuptools.find_packages(),
    include_package_data=True,
    zip_safe=True,
    # 依赖包
    install_requires=install_requires,
)
